#import "BarcodeView.h"
#import "WSDY1.h"

@implementation BarcodeView

- (id)initWithFrame:(NSRect)frameRect
{
	if((self = [super initWithFrame:frameRect]) != nil) 
	{
		WSDY1unlock("LXXXXX-XXXXX","WSDY1-XXXX-XXXXXXXX");
	}
	return self;
}

- (void)drawRect:(NSRect)rect
{
	[[NSColor whiteColor] setFill];	
	NSBezierPath *p = [NSBezierPath bezierPathWithRect:rect];
	[p fill];
	
	CGImageRef img = WSDY1createCode(72);
	NSImage *barcode = [self imageFromCGImageRef:img];
	
	[barcode drawAtPoint:	NSMakePoint(0,rect.size.height - [barcode size].height) 
							fromRect:NSMakeRect(0,0,[barcode size].width,[barcode size].height) 
							operation: NSCompositeCopy 
							fraction:1.0f];
							
	[barcode release];
	CGImageRelease(img);
}

- (void)setProperties
{
	[dataToEncode setStringValue: [NSString stringWithCString:WSDY1getDataToEncode()]];
	[moduleHeight setFloatValue:WSDY1getModuleHeight()];
	[moduleWidth setFloatValue:WSDY1getModuleWidth()];
	[gs1 setState: WSDY1getGs1() == true ? NSOnState : NSOffState];
	[humanReadableText setState: WSDY1getHumanReadable() == true ? NSOnState : NSOffState];	
}

// Actions

- (IBAction)gs1:(id)sender
{
	[self refresh:self];
}

- (IBAction)humanReadableText:(id)sender
{
	[self refresh:self];
}

- (IBAction)refresh:(id)sender
{
	char buffer[128];
	memset(buffer,0x00,128);
	
	[[dataToEncode stringValue] getCString:buffer maxLength:127];
	WSDY1setDataToEncode(buffer);
	
	WSDY1setModuleWidth([moduleWidth floatValue]);
	WSDY1setModuleHeight([moduleHeight floatValue]);
	
	WSDY1setHumanReadable([humanReadableText state] == NSOnState ? true : false);
	WSDY1setGs1([gs1 state] == NSOnState ? true : false);
	
	[self setNeedsDisplay:YES];
}

- (IBAction)reset:(id)sender
{
	WSDY1resetToDefaults();
	[self setProperties];
	[self setNeedsDisplay:YES];
}

- (IBAction)save:(id)sender
{
	NSSavePanel *panel = [NSSavePanel savePanel];
	[panel setRequiredFileType:@"png"];
	int runResult = [panel runModalForDirectory:nil file:@""];

	if (runResult == NSOKButton) 
	{
		CGImageRef img = WSDY1createCode(300);
		NSImage *barcode = [self imageFromCGImageRef:img];

		NSBitmapImageRep *rep = [NSBitmapImageRep imageRepWithData:[barcode TIFFRepresentation]]; 

		NSData *data = [rep representationUsingType: NSPNGFileType properties: nil];
		[data writeToFile: [panel filename] atomically: NO];

		[barcode release];
		CGImageRelease(img);
	}
}

// Helper

// c.f. http://developer.apple.com/documentation/Cocoa/Conceptual/CocoaDrawingGuide/Images/chapter_7_section_6.html

- (NSImage*)imageFromCGImageRef:(CGImageRef)image
{
    NSRect imageRect = NSMakeRect(0.0, 0.0, 0.0, 0.0);
    CGContextRef imageContext = nil;
    NSImage* newImage = nil;

    // Get the image dimensions.

    imageRect.size.height = CGImageGetHeight(image);
    imageRect.size.width = CGImageGetWidth(image);

    // Create a new image to receive the Quartz image data.

    newImage = [[NSImage alloc] initWithSize:imageRect.size]; 
    [newImage lockFocus];

    // Get the Quartz context and draw.

    imageContext = (CGContextRef)[[NSGraphicsContext currentContext] graphicsPort];
    CGContextDrawImage(imageContext, *(CGRect*)&imageRect, image);

    [newImage unlockFocus];
    return newImage;
}

@end
